using System;
using System.Runtime.InteropServices;

namespace BatteryMonitor
{
	/// <summary>
	/// BatteryServices is a class that wraps the GetSystemPowerStatus API in order
	/// to provide convenient access to battery status information such as the percent
	/// full of the battery, time remaining, etc.
	/// </summary>
	public class BatteryServices
	{
		public BatteryServices()	// As this class has no non-static members, there's nothing to initialize.
		{
		}

		// Here's the PInvoke wrapper for GetSystemPowerStatus.
		[DllImport("Kernel32")]
		private static extern Boolean GetSystemPowerStatus(SystemPowerStatus sps );

		/// <summary>
		/// Static method for returning a SystemPowerStatus structure.
		/// </summary>
		/// <returns></returns>
		public static SystemPowerStatus GetSystemPowerStatus()
		{
			SystemPowerStatus sps = new SystemPowerStatus();
			GetSystemPowerStatus( sps);
			return sps;
		}

		/// <summary>
		/// Symbols for the various power line status values.  Possible values are
		/// Offline (that is, battery power), Online (plugged into an AC socket),
		/// and Unknown.
		/// </summary>
		public enum ACLineStatus : byte
		{
			Offline = 0, Online = 1, Unknown = 255
		}

		/// <summary>
		/// Symbols for various battery power states.  Possible values are
		/// High, Low, Critical (that is, really low), Charging, NoSystemBattery,
		/// and Unknown.
		/// </summary>
		public enum BatteryFlag : byte
		{
			High = 1,
			Low = 2,
			Critical = 4,
			Charging = 8,
			NoSystemBattery = 128,
			Unknown = 255
		}

		/// <summary>
		/// This is a managed-code structure corresponding to the unmanaged structure
		/// that the GetSystemPowerStatus API is expecting.  Like all structures used
		/// with PInvokes, the fields must have the same types (or corresponding types)
		/// as their unmanaged counterparts, and in the same order.  The LayoutKind.Sequential
		/// attribute prevents the CLR from potentially re-arranging the fields in this
		/// structure.
		/// </summary>
		[StructLayout( LayoutKind.Sequential )]
			public class SystemPowerStatus
		{
			public ACLineStatus _ACLineStatus;
			public BatteryFlag  _BatteryFlag;
			public Byte     _BatteryLifePercent;
			public Byte     _Reserved1;
			public Int32    _BatteryLifeTime;
			public Int32    _BatteryFullLifeTime;
		}

		/// <summary>
		/// Returns an ACLineStatus value indicating the system's power line status.
		/// </summary>
		/// <returns></returns>
		public static ACLineStatus GetPowerLineStatus() 
		{
			SystemPowerStatus sps = GetSystemPowerStatus();
			return sps._ACLineStatus;
		}
		/// <summary>
		/// Returns a BatteryFlag value indicating the battery's status.
		/// </summary>
		/// <returns></returns>
		public static BatteryFlag GetBatteryFlag() 
		{
			SystemPowerStatus sps = GetSystemPowerStatus();
			return sps._BatteryFlag;
		}
		/// <summary>
		/// Returns the amount of battery power remaining, as a percentage from 0 to 100.
		/// </summary>
		/// <returns></returns>
		public static int GetBatteryLifePercentage() 
		{
			SystemPowerStatus sps = GetSystemPowerStatus();
			return (int)(sps._BatteryLifePercent);
		}
		/// <summary>
		/// Returns the number of seconds of battery power remaining, or -1 if that value
		/// is not available.
		/// </summary>
		/// <returns></returns>
		public static int GetBatteryLifeTime() 
		{
			SystemPowerStatus sps = GetSystemPowerStatus();
			return (int)(sps._BatteryLifeTime);
		}
		/// <summary>
		/// Returns the number of seconds of battery life for a full battery, or
		/// -1 if that value is not available.
		/// </summary>
		/// <returns></returns>
		public static int GetBatteryFullLifeTime() 
		{
			SystemPowerStatus sps = GetSystemPowerStatus();
			return (int)(sps._BatteryFullLifeTime);
		}
	}
}
