using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using BatteryMonitor;
using Microsoft.Win32;

namespace BatteryMonitor
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private BatteryServices.ACLineStatus LineStatus;
		private bool TrackingTimeLeft;
		private int TicksSinceStart, PercentAtStart;
		private string TimeRemaining;
		private BatteryServices.BatteryFlag Battery_Flag;
		private int BatteryPercent;
		private int BatteryLifeTime;
		private int BatteryFullTime;
		private bool MouseIsDown;
		private Point DragStart, MouseStart;
		private int CloseBoxLeft, CloseBoxRight, CloseBoxTop, CloseBoxBottom, CloseBoxSize;
		private int GraphLeft, GraphRight, GraphTop, GraphBottom, GraphHeight, GraphWidth;
		private int LowLeft, LowRight, MedLeft, MedRight, HighLeft, HighRight;
		private int WidthCache, HeightCache;
		private double OpacityCache;
		// GDI objects:
		private Pen FGLine;
		private Brush FGBrush, BGBrush, LowBrush, MedBrush, HighBrush;

		private System.Windows.Forms.Timer UpdateTimer;
		private System.ComponentModel.IContainer components;


		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// This block of stuff is just for initializing variables to satisfy C#'s
			// definite-assignment rule.
			TrackingTimeLeft = false;
			TicksSinceStart = 0;
			PercentAtStart = 0;
			TimeRemaining = (string)null;
			MouseIsDown = false;
			DragStart = new Point(0,0);
			MouseStart = new Point(0,0);

			// Here we get initial battery values, so that the first call to our paint routine
			// will work correctly, and turn on the timer so that we'll check battery values again
			// once per minute.  Ideally, the timer interval would be user-settable, but that
			// would require a lot more UI.
			UpdateTimer_Tick(this, (EventArgs)null);
			UpdateTimer.Enabled = true;

			// Here we calculate a bunch of constants that will be used in drawing.  We could
			// calculate these in the paint handler, but since they'll never change during the
			// lifetime of the program, we may as well just calculate them once and save them.
			CloseBoxSize = 7;								// this block is for the app's close box.
			CloseBoxRight = this.Width - 1;
			CloseBoxLeft = CloseBoxRight - CloseBoxSize;
			CloseBoxTop = 0;
			CloseBoxBottom = CloseBoxTop + CloseBoxSize;

			GraphLeft = 0;									// this block is the bounds of the
			GraphRight = this.Width - 1;					// actual battery graph.
			GraphTop = 0;
			GraphBottom = this.Height - 1;
			GraphWidth = GraphRight - GraphLeft;
			GraphHeight = GraphBottom - GraphTop;

			// These are threshhold amounts for low, medium, and high battery.
			// ideally, these would be user-settable, but that would require a lot more UI.
			double LowMin, LowMax, MedMin, MedMax, HighMin, HighMax;
			LowMin  = 0.00;		LowMax  = 0.25;
			MedMin  = 0.25;		MedMax  = 0.75;
			HighMin = 0.75;		HighMax = 1.00;

			LowLeft   = GraphLeft + (int)(LowMin * GraphWidth);	// this block contains the left
			LowRight  = GraphLeft + (int)(LowMax * GraphWidth);	// and right positions of each
			MedLeft   = GraphLeft + (int)(MedMin * GraphWidth);	// region of the battery graph.
			MedRight  = GraphLeft + (int)(MedMax * GraphWidth);
			HighLeft  = GraphLeft + (int)(HighMin * GraphWidth);
			HighRight = GraphLeft + (int)(HighMax * GraphWidth);

			// Here we define the GDI objects that will be used to paint our form.
			// Note that because GDI objects are unmanaged, we must explicitly dispose of them
			// when the application exits.
			FGLine = new Pen(this.ForeColor, 0.0f);
			FGBrush = new SolidBrush(this.ForeColor);
			BGBrush = new SolidBrush(this.BackColor);
			LowBrush = new SolidBrush(Color.Red);
			MedBrush = new SolidBrush(Color.Yellow);
			HighBrush = new SolidBrush(Color.Green);

			// Here we cache some initial values from the form.  You might well ask why we need
			// to cache the form's size.  It's because there seems to be some sort of bug with
			// forms that have FormBorderStyle=None, in which the actual size of the form changes
			// a little bit between when the constructor finishes and when the form gets painted.
			// On my system, the form's height changes by 6 pixels, which causes some dead space
			// underneath the battery graph.  To correct for this, we cache the size here and
			// reset it in the paint handler if we detect that the size is wrong.
			OpacityCache = this.Opacity;
			WidthCache = this.Width;
			HeightCache = this.Height;

			// Position the window at the bottom-right corner of the screen.  This overrides 
			// whatever default position that Windows would assign to the form.
			RelocateWindow();

			// Here we subscribe to some system events so that we can be notified when the computer
			// switches between AC power and battery power, and when the display settings change:
			SystemEvents.PowerModeChanged +=new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
			SystemEvents.DisplaySettingsChanged += new EventHandler(SystemEvents_DisplaySettingsChanged);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
					// dispose of our GDI objects:
					FGLine.Dispose();
					FGBrush.Dispose();
					BGBrush.Dispose();
					LowBrush.Dispose();
					MedBrush.Dispose();
					HighBrush.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.UpdateTimer = new System.Windows.Forms.Timer(this.components);
			// 
			// UpdateTimer
			// 
			this.UpdateTimer.Interval = 60000;
			this.UpdateTimer.Tick += new System.EventHandler(this.UpdateTimer_Tick);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(176, 21);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
			this.Name = "Form1";
			this.Opacity = 0.25;
			this.ShowInTaskbar = false;
			this.Text = "BatteryMonitor";
			this.TopMost = true;
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.Form1_MouseDown);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.Form1_MouseUp);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.Form1_MouseMove);
			this.MouseEnter += new System.EventHandler(this.Form1_MouseEnter);
			this.MouseLeave += new System.EventHandler(this.Form1_MouseLeave);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		/// <summary>
		/// This method refreshes the member variables holding battery status information, and
		/// updates the string saying how much time remaining the battery has.  Note that battery
		/// time remaining is only tracked (or calculated) when the computer is running on battery
		/// power.  There's no point in showing battery time when you're on AC power.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void UpdateTimer_Tick(object sender, System.EventArgs e)
		{
			// First, get new battery status information
			LineStatus = BatteryServices.GetPowerLineStatus();
			Battery_Flag = BatteryServices.GetBatteryFlag();
			BatteryPercent = BatteryServices.GetBatteryLifePercentage();
			BatteryLifeTime = BatteryServices.GetBatteryLifeTime();
			BatteryFullTime = BatteryServices.GetBatteryFullLifeTime();

			// Next, do some checks to see what situation we're in with respect to tracking
			// battery time remaining, and act accordingly.

			// If we're on battery power but we're _not_ tracking remaining battery lifetime, 
			// then we'd better start doing so:
			if(LineStatus == BatteryServices.ACLineStatus.Offline && TrackingTimeLeft == false) 
			{
				TrackingTimeLeft = true;
				TicksSinceStart = 0;
				PercentAtStart = BatteryPercent;
				TimeRemaining = "unknown";
			} 
			// If we're already tracking, then we just need to update the time-left string
			else if(LineStatus == BatteryServices.ACLineStatus.Offline && TrackingTimeLeft == true)
			{
				// keep track of the total amount of time we've been on battery power:
				TicksSinceStart += UpdateTimer.Interval; 

				if(BatteryLifeTime == -1)  // -1 means "unknown", so we have to calculate this ourselves
				{
					double DeltaPercent = PercentAtStart - BatteryPercent;
					if(DeltaPercent > 0.0) 
					{
						double ratioRemaining = (double)BatteryPercent / DeltaPercent;
						int SecondsRemaining = (int)(0.001 * TicksSinceStart * ratioRemaining);
						TimeRemaining = ((int)(SecondsRemaining/60)).ToString() + ":" + ((int)(SecondsRemaining%60)).ToString("D2");
					}
					else
						TimeRemaining = "unknown";
				}
				else // if BatteryLifeTime is not -1, then it holds the number of seconds of battery power left.
					TimeRemaining = ((int)(BatteryLifeTime/60)).ToString() + ":" + ((int)(BatteryLifeTime%60)).ToString("D2");
			}
			// The last option: either we're online or we can't tell what the line status is
			else if(LineStatus == BatteryServices.ACLineStatus.Online ||
					LineStatus == BatteryServices.ACLineStatus.Unknown ) 
			{
				TrackingTimeLeft = false;
				TimeRemaining = (string)null;
			}

			// Having updated the data, we now need to cause the form to re-paint itself so the
			// data can be shown to the user.  Invalidate() tells Windows that the form needs to
			// be repainted, and Update() forces Windows to send us the WM_PAINT message right now.
			this.Invalidate();
			this.Update();
		}

		/// <summary>
		/// By implementing our own paint method for the form, we can use GDI calls directly
		/// to draw whatever we like in our application's window, and thus we don't need any
		/// actual controls in order to show information to the user.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			// Here's where we detect if the form is the wrong size due to that FormBorderSize
			// bug described earlier, and re-size the form if that's the case.  Note that one
			// has to be careful about this, because resizing the form causes the form to be
			// repainted, so if you get this wrong you could end up in an infinite loop.  So
			// to be on the safe side, in this circumstance all we do is resize the form and 
			// not any actual painting.  When Form1_Paint gets called again in a few milliseconds,
			// then we'll go ahead and draw since the form will be the right size (or at least, 
			// it had better be!)
			if(this.Width != WidthCache || this.Height != HeightCache) 
			{
				this.Size = new Size(WidthCache, HeightCache);
				this.Invalidate();
				this.Update();
				return;
			}

			// OK, if we get here, it means the form is the right size and we can go ahead and draw.

			// calculate where in the battery graph the current value lies:
			int BatteryPosition = (int)((double)BatteryPercent / 100.0 * (double)GraphWidth);

			// Fill the inside of the battery graph with background color
			e.Graphics.FillRectangle(BGBrush, GraphLeft, GraphTop, GraphWidth, GraphHeight);

			// Draw the low, medium, and high battery regions.  First, the Low region:
			if(BatteryPosition >= LowRight)		// draw full low region
				e.Graphics.FillRectangle(LowBrush, LowLeft, GraphTop, (LowRight-LowLeft), GraphHeight);
			else								// draw partial low region
				e.Graphics.FillRectangle(LowBrush, LowLeft, GraphTop, (BatteryPosition-LowLeft), GraphHeight);

			// Next, draw the Medium region, which is a bit tricky since BatteryPosition can be either
			// to the right, inside, or to the left of the Medium region.
			if(BatteryPosition >= MedRight)		// On the right: draw full medium region
				e.Graphics.FillRectangle(MedBrush, MedLeft, GraphTop, (MedRight-MedLeft), GraphHeight);
			else if(BatteryPosition >= MedLeft)	// Inside: draw partial medium region
				e.Graphics.FillRectangle(MedBrush, MedLeft, GraphTop, (BatteryPosition-MedLeft), GraphHeight);
			// On the left: don't draw the medium region.

			// Finally, draw the High region.  There's no real difference between a partial and a full
			// High region, since full just means that BatteryPosition == HighRight, so we'll simply
			// always draw this region using the "partially full" logic.
			if(BatteryPosition >= HighLeft)
				e.Graphics.FillRectangle(HighBrush, HighLeft, GraphTop, (BatteryPosition-HighLeft) + 2, GraphHeight);

			// Draw border around graph region.  The graph just doesn't look good
			// without at least a small border.
			e.Graphics.DrawRectangle(FGLine, GraphLeft, GraphTop, GraphWidth, GraphHeight);

			// Draw the battery percentage / time remaining.
			string s;
			if(TrackingTimeLeft == true) 
				s = BatteryPercent.ToString() + " / " + TimeRemaining;
			else
				s = BatteryPercent.ToString();
			SizeF StringSize = e.Graphics.MeasureString(s, this.Font, GraphWidth, StringFormat.GenericDefault);
			int VPos = (int)(this.Height - StringSize.Height) / 2;
			e.Graphics.DrawString(s, this.Font, FGBrush, GraphLeft, VPos, StringFormat.GenericDefault);
		
			// Draw the close box
			e.Graphics.DrawRectangle(FGLine, CloseBoxLeft, CloseBoxTop, CloseBoxSize, CloseBoxSize);
			e.Graphics.DrawLine(FGLine, CloseBoxLeft, CloseBoxTop, CloseBoxRight, CloseBoxBottom);
			e.Graphics.DrawLine(FGLine, CloseBoxLeft, CloseBoxBottom, CloseBoxRight, CloseBoxTop);
		}

		/// <summary>
		///  Move the window to the bottom-right corner of the screen.  This method gets called during
		///  construction and during DisplaySettingsChanged event processing.
		/// </summary>
		private void RelocateWindow() 
		{
			Rectangle WA = Screen.GetWorkingArea(this);	// the usable portion of the screen, minus the taskbar and any docked toolbars.
			this.Location = new Point(WA.Width - this.WidthCache, WA.Height-this.HeightCache);	// have to use cached values or else this is wrong when called at startup.
		}

		/// <summary>
		/// Handler for mouse-down events.  Because we're a borderless, non-resizable form, we
		/// need to do our own mouse event processing so the user can drag the form around and
		/// close it.  This handler tracks whether the user has clicked (but not released) a mouse
		/// button, and stores the locations of the window and the click in order to support dragging.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			MouseIsDown = true;
			DragStart = this.Location;
			MouseStart = new Point(e.X, e.Y);
		}
		
		/// <summary>
		/// Handler for mouse-up events.  For us, a mouse-up either means that the user is done dragging
		/// the window, or has clicked inside the close box and wants to close the application.  We
		/// check the click location against the close box, and if it's inside, we exit the application.
		/// Otherwise, all we have to do is note the fact that the mouse isn't down, so that the Move
		/// handler won't move the window.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(MouseIsDown) 
			{
				if(e.X >= CloseBoxLeft && e.X <= CloseBoxRight &&
				   e.Y >= CloseBoxTop  && e.Y <= CloseBoxBottom) // then the user wants to exit
					Application.Exit();
			}
			MouseIsDown = false;
		}

		/// <summary>
		/// Handler for mouse move events.  All we do here is check to see whether the mouse button
		/// is down, and if so, calculate a new location for the window and move it there.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(MouseIsDown) 
			{
				int dx = e.X - MouseStart.X,
					dy = e.Y - MouseStart.Y;
				this.Location = new Point(this.Location.X + dx, this.Location.Y + dy);
			}
		}

		/// <summary>
		/// Handler for mouse enter events.  We use mouse enter and mouse leave to alter the opacity
		/// of the form; normally the form is mostly transparent so that the window is not obnoxious,
		/// but when the user mouses over the window, we want to make it fully opaque so that the user
		/// can accurately read the information in the window.  We cache the original opacity so that
		/// we can restore it when the user mouses out.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_MouseEnter(object sender, System.EventArgs e)
		{
			OpacityCache = this.Opacity;
			this.Opacity = 1.00;
		}
		
		/// <summary>
		/// Handler for mouse leave events.  All we do here is restore the form's opacity to whatever
		/// value was saved in the MouseEnter handler.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_MouseLeave(object sender, System.EventArgs e)
		{
			this.Opacity = OpacityCache;
		}

		/// <summary>
		/// Event handler for the DisplaySettingsChanged system event, which we get if the
		/// system changes from landscape to portrait mode, or if a tablet pc input panel
		/// pops up, or anything else happens that changes the working area of the screen.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SystemEvents_DisplaySettingsChanged(object sender, EventArgs e) 
		{
			// Just call RelocateWindow, which does the actual work.
			RelocateWindow();
		}

		/// <summary>
		/// Event handler for the PowerModeChanged system event, which we should get if the system switches
		/// between battery and AC power.  All we do is trigger another timer tick processing cycle.  The
		/// only point of doing this is so that when the user unplugs their laptop, they'll get an immediate
		/// update showing the battery time remaining (if that data is available, anyway.)
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e) 
		{
			UpdateTimer_Tick(this, (EventArgs)null);
		}
	}
}
